# Unit tests for where_am_i_presenter.py methods.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2.1 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-statements
# pylint: disable=protected-access
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
# pylint: disable=too-many-locals
# pylint: disable=too-many-lines

"""Unit tests for where_am_i_presenter.py methods."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

if TYPE_CHECKING:
    from .orca_test_context import OrcaTestContext

READ_CHAR_ATTRIBUTES_CMD = "readCharAttributesCommand"
PRESENT_SIZE_AND_POSITION_CMD = "presentSizeAndPositionCommand"
PRESENT_TITLE_CMD = "presentTitleCommand"
PRESENT_STATUS_BAR_CMD = "presentStatusBarCommand"
PRESENT_DEFAULT_BUTTON_CMD = "presentDefaultButtonCommand"
WHERE_AM_I_BASIC_CMD = "whereAmIBasicCommand"
WHERE_AM_I_DETAILED_CMD = "whereAmIDetailedCommand"
WHERE_AM_I_LINK_CMD = "whereAmILinkCommand"
WHERE_AM_I_SELECTION_CMD = "whereAmISelectionCommand"
LOCATION_NOT_FOUND_MSG = "Location not found"
DEFAULT_BUTTON_NOT_FOUND_MSG = "Default button not found"
STATUS_BAR_NOT_FOUND_FULL_MSG = "Status bar not found"
STATUS_BAR_NOT_FOUND_BRIEF_MSG = "No status bar"

@pytest.mark.unit
class TestWhereAmIPresenter:
    """Test WhereAmIPresenter class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext):
        """Returns all dependencies needed for WhereAmIPresenter testing."""

        additional_modules = [
            "orca.speech_and_verbosity_manager",
            "orca.ax_component",
            "orca.ax_text",
            "orca.ax_utilities",
        ]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        cmdnames_mock = essential_modules["orca.cmdnames"]
        cmdnames_mock.READ_CHAR_ATTRIBUTES = READ_CHAR_ATTRIBUTES_CMD
        cmdnames_mock.PRESENT_SIZE_AND_POSITION = PRESENT_SIZE_AND_POSITION_CMD
        cmdnames_mock.PRESENT_TITLE = PRESENT_TITLE_CMD
        cmdnames_mock.PRESENT_STATUS_BAR = PRESENT_STATUS_BAR_CMD
        cmdnames_mock.PRESENT_DEFAULT_BUTTON = PRESENT_DEFAULT_BUTTON_CMD
        cmdnames_mock.WHERE_AM_I_BASIC = WHERE_AM_I_BASIC_CMD
        cmdnames_mock.WHERE_AM_I_DETAILED = WHERE_AM_I_DETAILED_CMD
        cmdnames_mock.WHERE_AM_I_LINK = WHERE_AM_I_LINK_CMD
        cmdnames_mock.WHERE_AM_I_SELECTION = WHERE_AM_I_SELECTION_CMD

        keybindings_mock = essential_modules["orca.keybindings"]
        keybindings_mock.ORCA_MODIFIER_MASK = 2
        keybindings_mock.NO_MODIFIER_MASK = 0

        messages_mock = essential_modules["orca.messages"]
        messages_mock.BOLD = "bold"
        messages_mock.MISSPELLED = "misspelled"
        messages_mock.LOCATION_NOT_FOUND_FULL = LOCATION_NOT_FOUND_MSG
        messages_mock.LOCATION_NOT_FOUND_BRIEF = LOCATION_NOT_FOUND_MSG
        messages_mock.SIZE_AND_POSITION_FULL = "Size: %d by %d pixels, at %d, %d"
        messages_mock.SIZE_AND_POSITION_BRIEF = "%dx%d at %d,%d"
        messages_mock.DIALOG_NOT_IN_A = "Not in a dialog"
        messages_mock.DEFAULT_BUTTON_NOT_FOUND = DEFAULT_BUTTON_NOT_FOUND_MSG
        messages_mock.DEFAULT_BUTTON_IS_GRAYED = "Default button %s is grayed"
        messages_mock.DEFAULT_BUTTON_IS = "Default button is %s"
        messages_mock.STATUS_BAR_NOT_FOUND_FULL = STATUS_BAR_NOT_FOUND_FULL_MSG
        messages_mock.STATUS_BAR_NOT_FOUND_BRIEF = STATUS_BAR_NOT_FOUND_BRIEF_MSG
        messages_mock.NOT_ON_A_LINK = "Not on a link"
        messages_mock.NO_SELECTED_TEXT = "No selected text"
        messages_mock.SELECTED_TEXT_IS = "Selected text is %s"
        messages_mock.selected_items_count = test_context.Mock(
            return_value="2 of 5 items selected"
        )


        handler_mock = test_context.Mock()
        essential_modules["orca.input_event"].InputEventHandler = test_context.Mock(
            return_value=handler_mock
        )
        essential_modules["orca.input_event"].InputEvent = test_context.Mock()

        settings_manager_instance = test_context.Mock()

        def get_setting_side_effect(setting_name):
            if setting_name == "textAttributesToSpeak":
                return ["weight", "style", "underline"]
            return []

        settings_manager_instance.get_setting = test_context.Mock(
            side_effect=get_setting_side_effect
        )
        essential_modules["orca.settings_manager"].get_manager = test_context.Mock(
            return_value=settings_manager_instance
        )

        essential_modules["orca.settings"].repeatCharacterLimit = 4

        speech_verbosity_instance = test_context.Mock()
        speech_verbosity_instance.get_indentation_description = test_context.Mock(
            return_value=""
        )
        speech_verbosity_instance.adjust_for_presentation = test_context.Mock(
            return_value="adjusted text"
        )
        speech_verbosity_instance.adjust_for_digits = test_context.Mock(
            return_value="adjusted text"
        )
        essential_modules[
            "orca.speech_and_verbosity_manager"
        ].get_manager = test_context.Mock(return_value=speech_verbosity_instance)

        ax_component_mock = essential_modules["orca.ax_component"]
        rect_mock = test_context.Mock()
        rect_mock.width = 100
        rect_mock.height = 50
        rect_mock.x = 200
        rect_mock.y = 150
        ax_component_mock.AXComponent = test_context.Mock()
        ax_component_mock.AXComponent.get_rect = test_context.Mock(return_value=rect_mock)
        ax_component_mock.AXComponent.is_empty_rect = test_context.Mock(return_value=False)

        ax_text_mock = essential_modules["orca.ax_text"]
        ax_text_mock.AXText = test_context.Mock()
        ax_text_mock.AXText.get_text_attributes_at_offset = test_context.Mock(
            return_value=({"weight": "bold"}, 0, 5)
        )
        ax_text_mock.AXText.get_selected_text = test_context.Mock(
            return_value=("selected text", 0, 5)
        )
        ax_text_mock.AXText.get_all_supported_text_attributes = test_context.Mock()

        ax_text_attribute_instance = test_context.Mock()
        ax_text_attribute_instance.get_localized_name = test_context.Mock(
            return_value="Weight"
        )
        ax_text_attribute_instance.get_localized_value = test_context.Mock(
            return_value="Bold"
        )
        ax_text_attribute_instance.value_is_default = test_context.Mock(return_value=False)
        ax_text_attribute_instance.get_attribute_name = test_context.Mock(
            return_value="weight"
        )
        ax_text_mock.AXTextAttribute = test_context.Mock()
        ax_text_mock.AXTextAttribute.from_string = test_context.Mock(
            return_value=ax_text_attribute_instance
        )

        ax_utilities_mock = essential_modules["orca.ax_utilities"]
        ax_utilities_mock.AXUtilities = test_context.Mock()
        ax_utilities_mock.AXUtilities.get_default_button = test_context.Mock()
        ax_utilities_mock.AXUtilities.is_sensitive = test_context.Mock(return_value=True)
        ax_utilities_mock.AXUtilities.get_status_bar = test_context.Mock()
        ax_utilities_mock.AXUtilities.get_info_bar = test_context.Mock()
        ax_utilities_mock.AXUtilities.is_showing = test_context.Mock(return_value=True)
        ax_utilities_mock.AXUtilities.is_visible = test_context.Mock(return_value=True)
        ax_utilities_mock.AXUtilities.is_focused = test_context.Mock(return_value=True)
        ax_utilities_mock.AXUtilities.is_table_cell = test_context.Mock(return_value=False)
        ax_utilities_mock.AXUtilities.is_table_cell_or_header = test_context.Mock(
            return_value=False
        )
        ax_utilities_mock.AXUtilities.save_object_info_for_events = test_context.Mock()
        ax_utilities_mock.AXUtilities.is_list_item = test_context.Mock(return_value=False)
        ax_utilities_mock.AXUtilities.is_layout_only = test_context.Mock(return_value=False)

        return essential_modules

    def test_init(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.__init__ using OrcaTestContext for simpler mocking."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        presenter = WhereAmIPresenter()

        assert presenter._handlers is not None
        assert isinstance(presenter._handlers, dict)
        assert presenter._desktop_bindings is not None
        assert presenter._laptop_bindings is not None

        dbus_mock = deps["orca.dbus_service"]
        assert dbus_mock.get_remote_controller.call_count >= 1
        controller = dbus_mock.get_remote_controller.return_value
        controller.register_decorated_module.assert_called_with("WhereAmIPresenter", presenter)

    @pytest.mark.parametrize(
        "refresh,is_desktop,mock_empty",
        [
            pytest.param(True, True, False, id="desktop_refresh"),
            pytest.param(True, False, False, id="laptop_refresh"),
            pytest.param(False, True, True, id="desktop_empty"),
            pytest.param(False, False, True, id="laptop_empty"),
        ],
    )
    def test_get_bindings(
        self,
        test_context: OrcaTestContext,
        refresh: bool,
        is_desktop: bool,
        mock_empty: bool,
    ) -> None:
        """Test WhereAmIPresenter.get_bindings with various refresh and desktop settings."""

        self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        presenter = WhereAmIPresenter()

        if mock_empty:
            bindings_attr = (
                presenter._desktop_bindings if is_desktop else presenter._laptop_bindings
            )
            test_context.patch_object(bindings_attr, "is_empty", return_value=True)

        bindings = presenter.get_bindings(refresh=refresh, is_desktop=is_desktop)
        assert bindings is not None
        expected_bindings = (
            presenter._desktop_bindings if is_desktop else presenter._laptop_bindings
        )
        assert bindings == expected_bindings

    @pytest.mark.parametrize(
        "refresh,check_instance",
        [
            pytest.param(True, True, id="with_refresh"),
            pytest.param(False, False, id="without_refresh"),
        ],
    )
    def test_get_handlers(
        self,
        test_context: OrcaTestContext,
        refresh: bool,
        check_instance: bool,
    ) -> None:
        """Test WhereAmIPresenter.get_handlers with and without refresh."""

        self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        presenter = WhereAmIPresenter()
        handlers = presenter.get_handlers(refresh=refresh)
        assert handlers is not None

        if check_instance:
            assert isinstance(handlers, dict)
        else:
            assert handlers == presenter._handlers

    def test_setup_handlers(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter._setup_handlers."""

        self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        presenter = WhereAmIPresenter()
        presenter._setup_handlers()

        expected_handlers = [
            "readCharAttributesHandler",
            "presentSizeAndPositionHandler",
            "getTitleHandler",
            "getStatusBarHandler",
            "present_default_button",
            "whereAmIBasicHandler",
            "whereAmIDetailedHandler",
            "whereAmILinkHandler",
            "whereAmISelectionHandler",
        ]
        for handler_name in expected_handlers:
            assert handler_name in presenter._handlers

    @pytest.mark.parametrize(
        "attribute, value, expected",
        [
            pytest.param("weight", None, "", id="none_value"),
            pytest.param("weight", "bold", "bold", id="bold_weight"),
            pytest.param("weight", "700", "bold", id="numeric_weight"),
            pytest.param("spelling", "spelling", "misspelled", id="spelling"),
            pytest.param("style", "italic", "Weight: Bold", id="general"),
        ],
    )
    def test_localize_text_attribute_scenarios(
        self, test_context: OrcaTestContext, attribute: str, value: str | None, expected: str
    ) -> None:
        """Test WhereAmIPresenter._localize_text_attribute with various attribute scenarios."""

        self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        presenter = WhereAmIPresenter()
        result = presenter._localize_text_attribute(attribute, value)
        assert result == expected

    def test_present_character_attributes(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_character_attributes."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        mock_focus = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = mock_focus

        deps["orca.ax_text"].AXText.get_text_attributes_at_offset.return_value = (
            {"weight": "bold", "style": "italic"},
            0,
            5,
        )

        deps["orca.settings_manager"].get_manager.return_value.get_setting = test_context.Mock(
            return_value=["weight", "style", "underline"]
        )

        mock_script = test_context.Mock()
        test_context.patch_object(mock_script, "speak_message")
        presenter = WhereAmIPresenter()
        result = presenter.present_character_attributes(mock_script)
        assert result is True
        mock_script.speak_message.assert_called()
        assert mock_script.speak_message.call_count >= 1
        call_args = mock_script.speak_message.call_args_list
        assert len(call_args) > 0, "Expected at least one call to speak_message for attributes"

    def test_present_character_attributes_no_custom_attributes(
        self, test_context: OrcaTestContext
    ) -> None:
        """Test WhereAmIPresenter.present_character_attributes with no custom attributes."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        deps["orca.settings_manager"].get_manager.return_value.get_setting = test_context.Mock(
            return_value=[]
        )

        default_attr = test_context.Mock()
        default_attr.get_attribute_name.return_value = "weight"
        default_attr.value_is_default.return_value = False
        deps["orca.ax_text"].AXText.get_all_supported_text_attributes.return_value = [default_attr]
        mock_script = test_context.Mock()
        test_context.patch_object(mock_script, "speak_message")
        presenter = WhereAmIPresenter()
        result = presenter.present_character_attributes(mock_script)
        assert result is True
        mock_script.speak_message.assert_called()
        deps["orca.ax_text"].AXText.get_all_supported_text_attributes.assert_called_once()

    def test_present_size_and_position_flat_review(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_size_and_position in flat review mode."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        mock_script = test_context.Mock()
        flat_review_presenter = test_context.Mock()
        flat_review_presenter.is_active.return_value = True
        current_obj = test_context.Mock()
        flat_review_presenter.get_current_object.return_value = current_obj
        mock_script.get_flat_review_presenter.return_value = flat_review_presenter
        mock_script.present_message = test_context.Mock()

        rect = test_context.Mock()
        rect.width = 200
        rect.height = 100
        rect.x = 50
        rect.y = 75
        deps["orca.ax_component"].AXComponent.get_rect.return_value = rect
        deps["orca.ax_component"].AXComponent.is_empty_rect.return_value = False
        presenter = WhereAmIPresenter()
        result = presenter.present_size_and_position(mock_script)
        assert result is True
        mock_script.present_message.assert_called()

    def test_present_size_and_position_empty_rect(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_size_and_position with empty rectangle."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        mock_script = test_context.Mock()
        flat_review_presenter = test_context.Mock()
        flat_review_presenter.is_active.return_value = False
        mock_script.get_flat_review_presenter.return_value = flat_review_presenter
        mock_script.present_message = test_context.Mock()

        focus_obj = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = focus_obj

        deps["orca.ax_component"].AXComponent.is_empty_rect.return_value = True
        presenter = WhereAmIPresenter()
        result = presenter.present_size_and_position(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with(
            LOCATION_NOT_FOUND_MSG, LOCATION_NOT_FOUND_MSG)

    def test_present_title_valid_focus(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_title with valid focus object."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = focus_obj

        deps["orca.ax_object"].AXObject.is_dead.return_value = False

        mock_script = test_context.Mock()
        mock_script.speech_generator = test_context.Mock()
        mock_script.speech_generator.generate_window_title.return_value = [("Test Window", None)]
        mock_script.present_message = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter.present_title(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with("Test Window", voice=None)

    def test_present_title_dead_focus(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_title with dead focus object."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        dead_obj = test_context.Mock()
        active_window = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = dead_obj
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_active_window.return_value = active_window

        deps["orca.ax_object"].AXObject.is_dead.side_effect = [True, False]

        mock_script = test_context.Mock()
        mock_script.speech_generator = test_context.Mock()
        mock_script.speech_generator.generate_window_title.return_value = [("Active Window", None)]
        mock_script.present_message = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter.present_title(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with("Active Window", voice=None)

    def test_present_title_no_valid_object(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_title with no valid object."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        deps["orca.focus_manager"].get_manager.return_value.get_locus_of_focus.return_value = None
        deps["orca.focus_manager"].get_manager.return_value.get_active_window.return_value = None

        deps["orca.ax_object"].AXObject.is_dead.return_value = True
        mock_script = test_context.Mock()
        mock_script.present_message = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter.present_title(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with(LOCATION_NOT_FOUND_MSG)

    @pytest.mark.parametrize(
        "has_dialog, has_button, is_sensitive, button_name, expected_message",
        [
            pytest.param(True, True, True, "OK", "Default button is OK", id="button_success"),
            pytest.param(
                True, True, False, "OK", "Default button OK is grayed", id="button_grayed"
            ),
            pytest.param(False, False, False, None, "Not in a dialog", id="no_dialog"),
            pytest.param(True, False, False, None, DEFAULT_BUTTON_NOT_FOUND_MSG, id="no_button"),
        ],
    )
    def test_present_default_button_scenarios(
        self,
        test_context: OrcaTestContext,
        has_dialog: bool,
        has_button: bool,
        is_sensitive: bool,
        button_name: str | None,
        expected_message: str,
    ) -> None:
        """Test WhereAmIPresenter.present_default_button for various scenarios."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = focus_obj

        mock_script = test_context.Mock()
        dialog = test_context.Mock() if has_dialog else None
        mock_script.utilities.frame_and_dialog.return_value = (None, dialog)
        mock_script.present_message = test_context.Mock()

        if has_button:
            button = test_context.Mock()
            deps["orca.ax_utilities"].AXUtilities.get_default_button.return_value = button
            deps["orca.ax_utilities"].AXUtilities.is_sensitive.return_value = is_sensitive
            deps["orca.ax_object"].AXObject.get_name.return_value = button_name
        else:
            deps["orca.ax_utilities"].AXUtilities.get_default_button.return_value = None

        presenter = WhereAmIPresenter()
        result = presenter.present_default_button(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with(expected_message)

    def test_present_status_bar_with_info(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_status_bar with both status bar and info bar."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = focus_obj
        mock_script = test_context.Mock()
        frame = test_context.Mock()
        mock_script.utilities.frame_and_dialog.return_value = (frame, None)
        mock_script.present_object = test_context.Mock()

        statusbar = test_context.Mock()
        infobar = test_context.Mock()
        deps["orca.ax_utilities"].AXUtilities.get_status_bar.return_value = statusbar
        deps["orca.ax_utilities"].AXUtilities.get_info_bar.return_value = infobar
        deps["orca.ax_utilities"].AXUtilities.is_showing.return_value = True
        deps["orca.ax_utilities"].AXUtilities.is_visible.return_value = True
        presenter = WhereAmIPresenter()
        result = presenter.present_status_bar(mock_script)
        assert result is True
        assert mock_script.present_object.call_count == 2

    def test_present_status_bar_no_status_bar(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_status_bar with no status bar."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = focus_obj
        mock_script = test_context.Mock()
        frame = test_context.Mock()
        mock_script.utilities.frame_and_dialog.return_value = (frame, None)
        mock_script.present_object = test_context.Mock()
        mock_script.present_message = test_context.Mock()
        deps["orca.ax_utilities"].AXUtilities.get_status_bar.return_value = None
        deps["orca.ax_utilities"].AXUtilities.get_info_bar.return_value = None
        presenter = WhereAmIPresenter()
        result = presenter.present_status_bar(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with(
            STATUS_BAR_NOT_FOUND_FULL_MSG, STATUS_BAR_NOT_FOUND_BRIEF_MSG)

    def test_present_link_valid_link(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_link with valid link."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        link_obj = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = link_obj
        mock_script = test_context.Mock()
        mock_script.utilities.is_link.return_value = True
        presenter = WhereAmIPresenter()
        mock_do_where_am_i = test_context.patch_object(
            presenter, "_do_where_am_i", return_value=True
        )
        result = presenter.present_link(mock_script)
        assert result is True
        mock_do_where_am_i.assert_called_with(mock_script, True, link_obj)

    def test_present_link_not_link(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_link with non-link object."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        non_link_obj = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = non_link_obj
        mock_script = test_context.Mock()
        mock_script.utilities.is_link.return_value = False
        mock_script.present_message = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter.present_link(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with("Not on a link")

    def test_get_all_selected_text_spreadsheet_cell(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter._get_all_selected_text with spreadsheet cell."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        cell_obj = test_context.Mock()
        mock_script = test_context.Mock()
        mock_script.utilities.is_spreadsheet_cell.return_value = True

        deps["orca.ax_text"].AXText.get_selected_text.return_value = ("cell text", 0, 9)
        presenter = WhereAmIPresenter()
        result = presenter._get_all_selected_text(mock_script, cell_obj)
        assert result == "cell text"

    def test_get_all_selected_text_with_adjacent(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter._get_all_selected_text with adjacent text objects."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        text_obj = test_context.Mock()
        prev_obj = test_context.Mock()
        next_obj = test_context.Mock()
        mock_script = test_context.Mock()
        mock_script.utilities.is_spreadsheet_cell.return_value = False
        mock_script.utilities.find_previous_object.side_effect = [prev_obj, None]
        mock_script.utilities.find_next_object.side_effect = [next_obj, None]

        deps["orca.ax_text"].AXText.get_selected_text.side_effect = [
            ("current text", 0, 12),  # current object
            ("prev text", 0, 9),  # previous object
            ("next text", 0, 9),  # next object
        ]
        presenter = WhereAmIPresenter()
        result = presenter._get_all_selected_text(mock_script, text_obj)
        assert result == "prev text current text next text"

    def test_present_selected_text_no_focus(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_selected_text with no focus."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        deps["orca.focus_manager"].get_manager.return_value.get_locus_of_focus.return_value = None
        mock_script = test_context.Mock()
        test_context.patch_object(mock_script, "speak_message")
        presenter = WhereAmIPresenter()
        result = presenter.present_selected_text(mock_script)
        assert result is True
        mock_script.speak_message.assert_called_with(LOCATION_NOT_FOUND_MSG)

    def test_present_selected_text_with_text(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_selected_text with selected text."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps["focus_manager_instance"].get_locus_of_focus.return_value = focus_obj
        mock_script = test_context.Mock()
        test_context.patch_object(mock_script, "speak_message")

        presenter = WhereAmIPresenter()
        test_context.patch_object(
            presenter, "_get_all_selected_text", return_value="selected text"
        )

        manager = deps["orca.speech_and_verbosity_manager"].get_manager.return_value
        manager.get_indentation_description.return_value = "indent: 2"

        def mock_adjust_for_presentation(_obj, text) -> str:
            return f"processed {text}"

        def mock_adjust_for_digits(_obj, text) -> str:
            text_str = str(text) if text is not None else ""
            return text_str

        manager.adjust_for_presentation = mock_adjust_for_presentation
        manager.adjust_for_digits = mock_adjust_for_digits
        result = presenter.present_selected_text(mock_script)
        assert result is True
        expected_msg = "Selected text is indent: 2 processed selected text"
        mock_script.speak_message.assert_called_with(expected_msg)

    def test_present_selection_spreadsheet_handling(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_selection with spreadsheet cell range."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = focus_obj

        spreadsheet = test_context.Mock()
        deps["orca.ax_object"].AXObject.find_ancestor.return_value = spreadsheet
        mock_script = test_context.Mock()
        mock_script.utilities.is_spreadsheet_table = test_context.Mock(return_value=True)
        mock_script.utilities.speak_selected_cell_range.return_value = True
        presenter = WhereAmIPresenter()
        result = presenter.present_selection(mock_script)
        assert result is True
        mock_script.utilities.speak_selected_cell_range.assert_called_with(spreadsheet)

    def test_present_selection_container_selection(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_selection with selection container."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps["focus_manager_instance"].get_locus_of_focus.return_value = focus_obj

        deps["orca.ax_object"].AXObject.find_ancestor.return_value = None
        deps["orca.ax_object"].AXObject.get_name.side_effect = ["Item 1", "Item 2"]

        container = test_context.Mock()
        selected_items = [test_context.Mock(), test_context.Mock()]
        mock_script = test_context.Mock()
        mock_script.utilities.is_spreadsheet_table = test_context.Mock(return_value=False)
        mock_script.utilities.get_selection_container.return_value = container
        mock_script.utilities.selected_child_count.return_value = 2
        mock_script.utilities.selectable_child_count.return_value = 5
        mock_script.utilities.selected_children.return_value = selected_items
        mock_script.present_message = test_context.Mock()
        test_context.patch_object(mock_script, "speak_message")
        presenter = WhereAmIPresenter()
        result = presenter.present_selection(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with("2 of 5 items selected")
        mock_script.speak_message.assert_called_with("Item 1,Item 2")

    def test_present_selection_no_container(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.present_selection with no selection container."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps["focus_manager_instance"].get_locus_of_focus.return_value = focus_obj

        deps["orca.ax_object"].AXObject.find_ancestor.return_value = None
        mock_script = test_context.Mock()
        mock_script.utilities.is_spreadsheet_table = test_context.Mock(return_value=False)
        mock_script.utilities.get_selection_container.return_value = None
        presenter = WhereAmIPresenter()
        presenter.present_selected_text = test_context.Mock(return_value=True)
        result = presenter.present_selection(mock_script)
        assert result is True
        presenter.present_selected_text.assert_called_with(mock_script, None, focus_obj)

    def test_do_where_am_i_basic(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter._do_where_am_i with basic mode."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps["focus_manager_instance"].get_locus_of_focus.return_value = focus_obj

        deps["orca.ax_object"].AXObject.is_dead.return_value = False

        deps["orca.ax_utilities"].AXUtilities.is_focused.return_value = True
        deps["orca.ax_utilities"].AXUtilities.is_table_cell_or_header.return_value = False
        deps["orca.ax_utilities"].AXUtilities.is_list_item.return_value = False
        deps["orca.ax_utilities"].AXUtilities.is_layout_only.return_value = False
        mock_script = test_context.Mock()
        mock_script.spellcheck = None
        mock_script.present_object = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter._do_where_am_i(mock_script, basic_only=True)
        assert result is True
        mock_script.present_object.assert_called_once()

    def test_do_where_am_i_detailed(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter._do_where_am_i with detailed mode."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps["focus_manager_instance"].get_locus_of_focus.return_value = focus_obj

        deps["orca.ax_object"].AXObject.is_dead.return_value = False

        deps["orca.ax_utilities"].AXUtilities.is_focused.return_value = True
        mock_script = test_context.Mock()
        mock_script.spellcheck = None
        mock_script.present_object = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter._do_where_am_i(mock_script, basic_only=False)
        assert result is True
        call_args = mock_script.present_object.call_args
        assert call_args[1]["formatType"] == "detailedWhereAmI"

    def test_do_where_am_i_dead_object(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter._do_where_am_i with dead focus object."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        dead_obj = test_context.Mock()
        active_window = test_context.Mock()
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_locus_of_focus.return_value = dead_obj
        deps[
            "orca.focus_manager"
        ].get_manager.return_value.get_active_window.return_value = active_window

        deps["orca.ax_object"].AXObject.is_dead.side_effect = [True, False]

        deps["orca.ax_utilities"].AXUtilities.is_focused.return_value = True
        mock_script = test_context.Mock()
        mock_script.spellcheck = None
        mock_script.present_object = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter._do_where_am_i(mock_script)
        assert result is True

    def test_do_where_am_i_no_object(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter._do_where_am_i with no valid object."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        deps["orca.focus_manager"].get_manager.return_value.get_locus_of_focus.return_value = None
        deps["orca.focus_manager"].get_manager.return_value.get_active_window.return_value = None

        deps["orca.ax_object"].AXObject.is_dead.return_value = True
        mock_script = test_context.Mock()
        mock_script.spellcheck = None
        mock_script.present_message = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter._do_where_am_i(mock_script)
        assert result is True
        mock_script.present_message.assert_called_with(LOCATION_NOT_FOUND_MSG)

    def test_do_where_am_i_with_spellcheck(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter._do_where_am_i with active spellcheck."""

        deps = self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        focus_obj = test_context.Mock()
        deps["focus_manager_instance"].get_locus_of_focus.return_value = focus_obj

        deps["orca.ax_object"].AXObject.is_dead.return_value = False

        deps["orca.ax_utilities"].AXUtilities.is_focused.return_value = True

        spellcheck_mock = test_context.Mock()
        spellcheck_mock.is_active.return_value = True
        spellcheck_mock.present_error_details = test_context.Mock()
        mock_script = test_context.Mock()
        mock_script.spellcheck = spellcheck_mock
        mock_script.present_object = test_context.Mock()
        presenter = WhereAmIPresenter()
        result = presenter._do_where_am_i(mock_script, basic_only=False)
        assert result is True
        spellcheck_mock.present_error_details.assert_called_with(True)

    def test_where_am_i_basic(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.where_am_i_basic."""

        self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        mock_script = test_context.Mock()
        presenter = WhereAmIPresenter()
        mock_do_where_am_i = test_context.patch_object(
            presenter, "_do_where_am_i", return_value=True
        )
        result = presenter.where_am_i_basic(mock_script)
        assert result is True
        mock_do_where_am_i.assert_called_with(mock_script, notify_user=True)

    def test_where_am_i_detailed(self, test_context: OrcaTestContext) -> None:
        """Test WhereAmIPresenter.where_am_i_detailed."""

        self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import WhereAmIPresenter

        mock_script = test_context.Mock()
        mock_script.interrupt_presentation = test_context.Mock()
        presenter = WhereAmIPresenter()
        mock_do_where_am_i = test_context.patch_object(
            presenter, "_do_where_am_i", return_value=True
        )
        result = presenter.where_am_i_detailed(mock_script)
        assert result is True
        mock_script.interrupt_presentation.assert_called_once()
        mock_do_where_am_i.assert_called_with(mock_script, False, notify_user=True)

    def test_get_presenter(self, test_context: OrcaTestContext) -> None:
        """Test get_presenter function."""

        self._setup_dependencies(test_context)
        from orca.where_am_i_presenter import get_presenter

        presenter = get_presenter()
        assert presenter is not None
        presenter2 = get_presenter()
        assert presenter is presenter2



# Unit tests for debugging_tools_manager.py methods.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-statements
# pylint: disable=protected-access
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
# pylint: disable=too-many-locals

"""Unit tests for debugging_tools_manager.py methods."""

from __future__ import annotations
from typing import TYPE_CHECKING
from unittest.mock import call
import gi
import pytest

gi.require_version("Atspi", "2.0")
from gi.repository import Atspi

if TYPE_CHECKING:
    from .orca_test_context import OrcaTestContext
    from unittest.mock import MagicMock

@pytest.mark.unit
class TestDebuggingToolsManager:
    """Test DebuggingToolsManager class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Set up mocks for debugging_tools_manager dependencies."""

        additional_modules = [
            "orca.orca_platform",
            "orca.ax_utilities",
            "orca.ax_utilities_debugging",
            "faulthandler",
            "subprocess",
        ]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        debug_mock = essential_modules["orca.debug"]
        debug_mock.debugFile = None
        debug_mock.debugLevel = 1000
        debug_mock.LEVEL_ALL = 0
        debug_mock.LEVEL_INFO = 800
        debug_mock.LEVEL_WARNING = 900
        debug_mock.LEVEL_SEVERE = 1000
        debug_mock.LEVEL_OFF = 10000
        debug_mock.print_message = test_context.Mock()

        cmdnames_mock = essential_modules["orca.cmdnames"]
        cmdnames_mock.DEBUG_CYCLE_LEVEL = "cycleDebugLevel"
        cmdnames_mock.DEBUG_CLEAR_ATSPI_CACHE_FOR_APPLICATION = "clearAtSpiCache"
        cmdnames_mock.DEBUG_CAPTURE_SNAPSHOT = "captureSnapshot"

        input_event_mock = essential_modules["orca.input_event"]
        input_event_handler_mock = test_context.Mock()
        input_event_mock.InputEventHandler = test_context.Mock(
            return_value=input_event_handler_mock
        )
        input_event_mock.InputEvent = test_context.Mock

        keybindings_mock = essential_modules["orca.keybindings"]
        key_bindings_instance = test_context.Mock()
        key_bindings_instance.is_empty = test_context.Mock(return_value=True)
        key_bindings_instance.remove_key_grabs = test_context.Mock(return_value=None)
        key_bindings_instance.add = test_context.Mock(return_value=None)
        keybindings_mock.KeyBindings = test_context.Mock(return_value=key_bindings_instance)
        keybindings_mock.KeyBinding = test_context.Mock()
        keybindings_mock.DEFAULT_MODIFIER_MASK = 1
        keybindings_mock.NO_MODIFIER_MASK = 0

        messages_mock = essential_modules["orca.messages"]
        messages_mock.DEBUG_CLEAR_CACHE_FAILED = "Failed to clear cache"
        messages_mock.DEBUG_CLEAR_CACHE = "Cache cleared"
        messages_mock.DEBUG_CAPTURE_SNAPSHOT_START = "Snapshot started"
        messages_mock.DEBUG_CAPTURE_SNAPSHOT_END = "Snapshot ended"

        orca_platform_mock = essential_modules["orca.orca_platform"]
        orca_platform_mock.version = "3.50.0"
        orca_platform_mock.revision = "abc123"

        settings_manager_mock = essential_modules["orca.settings_manager"]
        settings_manager_instance = test_context.Mock()
        settings_manager_instance.override_key_bindings = test_context.Mock(
            return_value=key_bindings_instance
        )
        settings_manager_instance.get_overridden_settings_for_debugging = test_context.Mock(
            return_value={}
        )
        settings_manager_instance.customized_settings = {}
        settings_manager_mock.get_manager = test_context.Mock(
            return_value=settings_manager_instance
        )

        focus_manager_mock = essential_modules["orca.focus_manager"]
        focus_manager_instance = test_context.Mock()
        focus_manager_instance.get_active_mode_and_object_of_interest = test_context.Mock(
            return_value=("focus", test_context.Mock(spec=Atspi.Accessible))
        )
        focus_manager_mock.get_manager = test_context.Mock(return_value=focus_manager_instance)

        ax_object_mock = essential_modules["orca.ax_object"]
        ax_object_mock.get_name = test_context.Mock(return_value="TestApp")
        ax_object_mock.clear_cache = test_context.Mock()

        ax_utilities_mock = essential_modules["orca.ax_utilities"]
        ax_utilities_mock.get_application = test_context.Mock(
            return_value=test_context.Mock(spec=Atspi.Accessible)
        )
        test_apps = [
            test_context.Mock(spec=Atspi.Accessible),
            test_context.Mock(spec=Atspi.Accessible),
        ]
        ax_utilities_mock.get_all_applications = test_context.Mock(return_value=test_apps)
        ax_utilities_mock.get_process_id = test_context.Mock(return_value=12345)
        ax_utilities_mock.is_application_unresponsive = test_context.Mock(return_value=False)

        ax_utilities_debugging_mock = essential_modules["orca.ax_utilities_debugging"]
        ax_utilities_debugging_mock.as_string = test_context.Mock(return_value="debug_info")

        faulthandler_mock = essential_modules["faulthandler"]
        faulthandler_mock.enable = test_context.Mock()

        subprocess_mock = essential_modules["subprocess"]
        subprocess_mock.getoutput = test_context.Mock(return_value="test_command")
        subprocess_mock.SubprocessError = Exception

        essential_modules["input_event_handler"] = input_event_handler_mock
        essential_modules["key_bindings_instance"] = key_bindings_instance
        essential_modules["settings_manager_instance"] = settings_manager_instance
        essential_modules["focus_manager_instance"] = focus_manager_instance

        return essential_modules

    @pytest.mark.parametrize(
        "debug_file_config",
        [
            pytest.param(None, id="basic"),
            pytest.param("mock_file", id="with_debug_file"),
            pytest.param(None, id="without_debug_file"),
        ],
    )
    def test_init_scenarios(
        self,
        test_context: OrcaTestContext,
        debug_file_config: str | None,
    ) -> None:
        """Test DebuggingToolsManager.__init__ with various scenarios."""
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)

        if debug_file_config == "mock_file":
            debug_file_mock = test_context.Mock()
            debug_file_mock.name = "/tmp/debug.log"
            essential_modules["orca.debug"].debugFile = debug_file_mock
            test_context.patch("os.path.exists", return_value=True)
        elif debug_file_config is None:
            essential_modules["orca.debug"].debugFile = None

        from orca.debugging_tools_manager import DebuggingToolsManager

        manager = DebuggingToolsManager()

        assert manager is not None
        assert manager._handlers is not None
        assert manager._bindings is not None
        assert isinstance(manager._handlers, dict)

    @pytest.mark.parametrize(
        "method_name,refresh,is_empty,expects_remove_grabs,expected_type",
        [
            pytest.param("get_bindings", False, True, False, None, id="bindings_no_refresh_empty"),
            pytest.param("get_bindings", True, True, True, None, id="bindings_refresh_empty"),
            pytest.param(
                "get_bindings", False, False, False, None, id="bindings_no_refresh_not_empty"
            ),
            pytest.param("get_handlers", False, False, False, dict, id="handlers_no_refresh"),
            pytest.param("get_handlers", True, False, False, dict, id="handlers_refresh"),
        ],
    )
    def test_get_methods(
        self,
        test_context: OrcaTestContext,
        method_name: str,
        refresh: bool,
        is_empty: bool,
        expects_remove_grabs: bool,
        expected_type: type | None,
    ) -> None:
        """Test DebuggingToolsManager.get_bindings and get_handlers methods."""
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.debugging_tools_manager import DebuggingToolsManager

        manager = DebuggingToolsManager()

        if method_name == "get_bindings" and not is_empty:
            test_context.patch_object(manager._bindings, "is_empty", return_value=False)

        method = getattr(manager, method_name)
        if method_name == "get_bindings":
            result = method(refresh=refresh, is_desktop=not refresh)
            if not is_empty and not refresh:
                assert result == manager._bindings
        else:
            result = method(refresh=refresh)

        assert result is not None
        if expected_type:
            assert isinstance(result, expected_type)

        if refresh:
            if method_name == "get_bindings":
                essential_modules["orca.keybindings"].KeyBindings.assert_called()
            essential_modules["orca.debug"].print_message.assert_called()

        if expects_remove_grabs:
            key_bindings_instance = essential_modules["key_bindings_instance"]
            key_bindings_instance.remove_key_grabs.assert_called_once()

    @pytest.mark.parametrize(
        "setup_method,expected_attrs",
        [
            pytest.param(
                "_setup_handlers",
                ["cycleDebugLevelHandler", "clear_atspi_app_cache", "capture_snapshot"],
                id="setup_handlers",
            ),
            pytest.param("_setup_bindings", [], id="setup_bindings"),
        ],
    )
    def test_setup_methods(
        self,
        test_context: OrcaTestContext,
        setup_method: str,
        expected_attrs: list[str],
    ) -> None:
        """Test DebuggingToolsManager setup methods."""
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.debugging_tools_manager import DebuggingToolsManager

        manager = DebuggingToolsManager()
        method = getattr(manager, setup_method)
        method()

        if setup_method == "_setup_handlers":
            for attr in expected_attrs:
                assert attr in manager._handlers
            assert essential_modules["orca.input_event"].InputEventHandler.call_count >= 3
        elif setup_method == "_setup_bindings":
            essential_modules["orca.keybindings"].KeyBindings.assert_called()
            settings_instance = essential_modules["settings_manager_instance"]
            settings_instance.override_key_bindings.assert_called_once()

    @pytest.mark.parametrize(
        "initial_level,expected_level,expected_message,expected_brief,has_event",
        [
            pytest.param(0, 800, "Debug level info.", "info", False, id="all_to_info"),
            pytest.param(10000, 0, "Debug level all.", "all", False, id="off_to_all"),
            pytest.param(
                800, 900, "Debug level warning.", "warning", True, id="info_to_warning_with_event"
            ),
            pytest.param(900, 1000, "Debug level severe.", "severe", False, id="warning_to_severe"),
            pytest.param(1000, 10000, "Debug level off.", "off", False, id="severe_to_off"),
        ],
    )
    def test_cycle_debug_level_comprehensive(
        self,
        test_context: OrcaTestContext,
        initial_level: int,
        expected_level: int,
        expected_message: str,
        expected_brief: str,
        has_event: bool,
    ) -> None:
        """Test DebuggingToolsManager._cycle_debug_level cycling through all levels."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        essential_modules["orca.debug"].debugLevel = initial_level
        from orca.debugging_tools_manager import DebuggingToolsManager

        manager = DebuggingToolsManager()
        script_mock = test_context.Mock()
        event_mock = test_context.Mock() if has_event else None

        result = manager._cycle_debug_level(script_mock, event_mock)
        assert result
        assert essential_modules["orca.debug"].debugLevel == expected_level
        script_mock.present_message.assert_called_with(expected_message, expected_brief)

    @pytest.mark.parametrize(
        "test_scenario,focus_object,application_object,expected_message,"
        "expects_clear_cache,expects_debug_call",
        [
            pytest.param(
                "success",
                "mock_obj",
                "mock_app",
                "Cache cleared",
                True,
                False,
                id="successful_cache_clear",
            ),
            pytest.param(
                "null_object",
                None,
                "mock_app",
                "Failed to clear cache",
                False,
                True,
                id="null_object_failure",
            ),
            pytest.param(
                "null_application",
                "mock_obj",
                None,
                "Failed to clear cache",
                False,
                True,
                id="null_application_failure",
            ),
        ],
    )
    def test_clear_atspi_app_cache_comprehensive(
        self,
        test_context: OrcaTestContext,
        test_scenario: str,
        focus_object: str | None,
        application_object: str | None,
        expected_message: str,
        expects_clear_cache: bool,
        expects_debug_call: bool,
    ) -> None:
        """Test DebuggingToolsManager._clear_atspi_app_cache with various scenarios."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.debugging_tools_manager import DebuggingToolsManager

        manager = DebuggingToolsManager()
        script_mock = test_context.Mock()

        test_obj = test_context.Mock(spec=Atspi.Accessible) if focus_object else None
        test_app = test_context.Mock(spec=Atspi.Accessible) if application_object else None

        if test_scenario == "null_object":
            focus_instance = essential_modules["focus_manager_instance"]
            focus_instance.get_active_mode_and_object_of_interest.return_value = ("focus", None)
        else:
            mock_focus_instance = test_context.Mock()
            mock_focus_instance.get_active_mode_and_object_of_interest.return_value = (
                "focus",
                test_obj,
            )
            test_context.patch(
                "orca.debugging_tools_manager.focus_manager.get_manager",
                return_value=mock_focus_instance
            )

        test_context.patch(
            "orca.debugging_tools_manager.AXUtilities.get_application", return_value=test_app
        )

        mock_clear_cache = test_context.patch(
            "orca.debugging_tools_manager.AXObject.clear_cache"
        )
        mock_debug_print = test_context.patch(
            "orca.debugging_tools_manager.debug.print_message"
        )

        result = manager._clear_atspi_app_cache(script_mock)
        assert result
        script_mock.present_message.assert_called_with(expected_message)

        if expects_clear_cache:
            mock_clear_cache.assert_called_once_with(
                test_app, recursive=True, reason="User request."
            )
        else:
            mock_clear_cache.assert_not_called()

        if expects_debug_call:
            if test_scenario == "null_object":
                debug_calls = [
                    call
                    for call in essential_modules["orca.debug"].print_message.call_args_list
                    if "Cannot clear cache on null object" in str(call)
                ]
                assert len(debug_calls) >= 1
            else:
                mock_debug_print.assert_called()

    @pytest.mark.parametrize(
        "test_method,scenario_data",
        [
            pytest.param(
                "_capture_snapshot",
                {
                    "original_level": 1,
                    "expected_calls": ["Snapshot started", "Snapshot ended"],
                    "expects_debug": True,
                    "expects_print_running": True,
                },
                id="capture_snapshot",
            ),
            pytest.param(
                "_get_running_applications_as_string_iter",
                {
                    "is_command_line": True,
                    "app_count": 2,
                    "expected_length": 3,
                    "header_contains": "Desktop has 2 app(s):",
                },
                id="get_running_apps_cli",
            ),
            pytest.param(
                "_get_running_applications_as_string_iter",
                {
                    "is_command_line": False,
                    "app_count": 1,
                    "expected_length": 2,
                    "header_contains": "DEBUGGING TOOLS MANAGER:",
                },
                id="get_running_apps_not_cli",
            ),
        ],
    )
    def test_debug_utility_methods(
        self, test_context: OrcaTestContext, test_method: str, scenario_data: dict
    ) -> None:
        """Test DebuggingToolsManager debug utility methods."""
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.debugging_tools_manager import DebuggingToolsManager

        manager = DebuggingToolsManager()
        script_mock = test_context.Mock()

        if test_method == "_capture_snapshot":
            essential_modules["orca.debug"].debugLevel = scenario_data["original_level"]
            mock_print_running = test_context.Mock()
            test_context.patch_object(
                manager, "print_running_applications", side_effect=mock_print_running
            )

            result = manager._capture_snapshot(script_mock)
            assert result
            assert essential_modules["orca.debug"].debugLevel == scenario_data["original_level"]

            expected_calls = [call(msg) for msg in scenario_data["expected_calls"]]
            script_mock.present_message.assert_has_calls(expected_calls)

            if scenario_data["expects_debug"]:
                essential_modules["orca.debug"].print_message.assert_called()
            if scenario_data["expects_print_running"]:
                mock_print_running.assert_called_once()

        elif test_method == "_get_running_applications_as_string_iter":
            app_count = scenario_data["app_count"]
            test_apps = [test_context.Mock(spec=Atspi.Accessible) for _ in range(app_count)]

            test_context.patch(
                "orca.debugging_tools_manager.AXUtilities.get_all_applications",
                return_value=test_apps,
            )
            test_context.patch(
                "orca.debugging_tools_manager.AXUtilities.get_process_id", return_value=12345
            )
            test_context.patch(
                "orca.debugging_tools_manager.AXUtilities.is_application_unresponsive",
                return_value=False,
            )
            test_context.patch(
                "orca.debugging_tools_manager.AXObject.get_name", return_value="TestApp"
            )
            test_context.patch(
                "orca.debugging_tools_manager.subprocess.getoutput", return_value="test_command"
            )

            result_list = list(
                manager._get_running_applications_as_string_iter(
                    is_command_line=scenario_data["is_command_line"]
                )
            )
            assert len(result_list) == scenario_data["expected_length"]
            assert scenario_data["header_contains"] in result_list[0]

    @pytest.mark.parametrize(
        "force,is_command_line,debug_level,expects_method_call,expects_debug,expects_stdout",
        [
            pytest.param(True, False, 1000, True, True, False, id="force_true_not_cli"),
            pytest.param(False, True, 1000, True, False, True, id="no_force_cli"),
            pytest.param(False, False, 900, False, False, False, id="debug_level_too_low"),
        ],
    )
    def test_print_running_applications(
        self,
        test_context: OrcaTestContext,
        force: bool,
        is_command_line: bool,
        debug_level: int,
        expects_method_call: bool,
        expects_debug: bool,
        expects_stdout: bool,
        capsys,
    ) -> None:
        """Test DebuggingToolsManager.print_running_applications."""
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        essential_modules["orca.debug"].debugLevel = debug_level
        from orca.debugging_tools_manager import DebuggingToolsManager

        manager = DebuggingToolsManager()
        expected_output = ["Header", "App 1", "App 2"]
        mock_get_apps = test_context.patch_object(
            manager, "_get_running_applications_as_string_iter", return_value=expected_output
        )

        manager.print_running_applications(force=force, is_command_line=is_command_line)

        if not expects_method_call:
            mock_get_apps.assert_not_called()
        else:
            mock_get_apps.assert_called_once()
            if expects_debug:
                expected_calls = [call(1000, line, True) for line in expected_output]
                essential_modules["orca.debug"].print_message.assert_has_calls(expected_calls)
            if expects_stdout:
                captured = capsys.readouterr()
                for line in expected_output:
                    assert line in captured.out

    @pytest.mark.parametrize(
        "is_command_line,revision,session_type,session_desktop,"
        "expects_stdout,expects_revision,expects_session",
        [
            pytest.param(
                True,
                "abc123",
                "wayland",
                "gnome",
                True,
                True,
                True,
                id="cli_with_revision_and_session",
            ),
            pytest.param(
                False,
                "abc123",
                "x11",
                "",
                False,
                True,
                True,
                id="not_cli_x11_no_desktop",
            ),
            pytest.param(
                False,
                None,
                "wayland",
                "gnome",
                False,
                False,
                True,
                id="not_cli_no_revision",
            ),
            pytest.param(
                False,
                "abc123",
                None,
                None,
                False,
                True,
                False,
                id="not_cli_no_session_env",
            ),
        ],
    )
    def test_print_session_details_comprehensive(  # pylint: disable=too-many-branches
        self,
        test_context: OrcaTestContext,
        is_command_line: bool,
        revision: str | None,
        session_type: str | None,
        session_desktop: str | None,
        expects_stdout: bool,
        expects_revision: bool,
        expects_session: bool,
        capsys,
    ) -> None:
        """Test DebuggingToolsManager.print_session_details with various scenarios."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        if revision is None:
            essential_modules["orca.orca_platform"].revision = None

        mock_atspi = test_context.Mock()
        mock_atspi.get_version.return_value = (2, 48, 0)
        test_context.patch("gi.repository.Atspi", new=mock_atspi)

        env_patches = {}
        remove_vars = []

        if session_type:
            env_patches["XDG_SESSION_TYPE"] = session_type
        else:
            remove_vars.append("XDG_SESSION_TYPE")

        if session_desktop:
            env_patches["XDG_SESSION_DESKTOP"] = session_desktop
        else:
            remove_vars.append("XDG_SESSION_DESKTOP")

        test_context.patch_env(env_patches, remove_vars=remove_vars)

        from orca.debugging_tools_manager import DebuggingToolsManager

        manager = DebuggingToolsManager()
        manager.print_session_details(is_command_line=is_command_line)

        if expects_stdout:
            captured = capsys.readouterr()
            assert "Orca version 3.50.0" in captured.out
            assert "AT-SPI2 version: 2.48.0" in captured.out

            if expects_revision:
                assert "rev abc123" in captured.out

            if expects_session:
                if session_desktop:
                    assert f"Session: {session_type} {session_desktop}" in captured.out
                else:
                    assert f"Session: {session_type}" in captured.out
        else:
            session_calls = [
                call
                for call in essential_modules["orca.debug"].print_message.call_args_list
                if "Orca version" in str(call)
            ]
            assert len(session_calls) >= 1
            call_args = session_calls[0]

            if not is_command_line:
                assert "DEBUGGING TOOLS MANAGER:" in call_args[0][1]

            assert "Orca version 3.50.0" in call_args[0][1]
            assert "AT-SPI2 version: 2.48.0" in call_args[0][1]

            if expects_revision:
                assert "rev abc123" in call_args[0][1]
            else:
                assert "rev" not in call_args[0][1]

            if expects_session:
                if session_desktop:
                    assert f"Session: {session_type} {session_desktop}" in call_args[0][1]
                else:
                    assert f"Session: {session_type}" in call_args[0][1]
            else:
                assert "Session:" not in call_args[0][1]

    def test_get_manager_singleton(self, test_context: OrcaTestContext) -> None:
        """Test debugging_tools_manager.get_manager singleton behavior."""
        self._setup_dependencies(test_context)
        from orca import debugging_tools_manager

        manager1 = debugging_tools_manager.get_manager()
        manager2 = debugging_tools_manager.get_manager()

        assert manager1 is manager2
        assert manager1._handlers is not None
        assert manager1._bindings is not None
